==> llvm-toolchain-19_19.1.7.orig.tar.xz: llvm-toolchain-19_19.1.7/libcxx/test/std/ranges/range.factories/range.iota.view/views_iota.pass.cpp <==
//===----------------------------------------------------------------------===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//

// UNSUPPORTED: c++03, c++11, c++14, c++17

// views::iota

#include <cassert>
#include <concepts>
#include <ranges>
#include <type_traits>

#include "test_macros.h"
#include "types.h"

template<class T, class U>
constexpr void testType(U u) {
  // Test that this generally does the right thing.
  // Test with only one argument.
  {
    assert(*std::views::iota(T(0)).begin() == T(0));
  }
  {
    const auto io = std::views::iota(T(10));
    assert(*io.begin() == T(10));
  }
  // Test with two arguments.
  {
    assert(*std::views::iota(T(0), u).begin() == T(0));
  }
  {
    const auto io = std::views::iota(T(10), u);
    assert(*io.begin() == T(10));
  }
  // Test that we return the correct type.
  {
    ASSERT_SAME_TYPE(decltype(std::views::iota(T(10))), std::ranges::iota_view<T>);
    ASSERT_SAME_TYPE(decltype(std::views::iota(T(10), u)), std::ranges::iota_view<T, U>);
  }
}

struct X {};

constexpr bool test() {
  testType<SomeInt>(SomeInt(10));
  testType<SomeInt>(IntComparableWith(SomeInt(10)));
  testType<signed long>(IntComparableWith<signed long>(10));
  testType<unsigned long>(IntComparableWith<unsigned long>(10));
  testType<int>(IntComparableWith<int>(10));
  testType<int>(int(10));
  testType<unsigned>(unsigned(10));
  testType<unsigned>(IntComparableWith<unsigned>(10));
  testType<short>(short(10));
  testType<short>(IntComparableWith<short>(10));
  testType<unsigned short>(IntComparableWith<unsigned short>(10));

  {
    static_assert( std::is_invocable_v<decltype(std::views::iota), int>);
    static_assert(!std::is_invocable_v<decltype(std::views::iota), X>);
    static_assert( std::is_invocable_v<decltype(std::views::iota), int, int>);
    static_assert(!std::is_invocable_v<decltype(std::views::iota), int, X>);
  }
  {
    static_assert(std::same_as<decltype(std::views::iota), decltype(std::ranges::views::iota)>);
  }

  return true;
}

int main(int, char**) {
  test();
  static_assert(test());

  return 0;
}

==> llvm-toolchain-22_22.1.0~rc2.orig.tar.xz: llvm-toolchain-22-22.1.0~rc2/libcxx/test/std/ranges/range.factories/range.iota.view/views_iota.pass.cpp <==
//===----------------------------------------------------------------------===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//

// UNSUPPORTED: c++03, c++11, c++14, c++17

// views::iota

#include <cassert>
#include <concepts>
#include <ranges>
#include <type_traits>

#include "test_macros.h"
#include "types.h"

template<class T, class U>
constexpr void testType(U u) {
  // Test that this generally does the right thing.
  // Test with only one argument.
  {
    assert(*std::views::iota(T(0)).begin() == T(0));
  }
  {
    const auto io = std::views::iota(T(10));
    assert(*io.begin() == T(10));
  }
  // Test with two arguments.
  {
    assert(*std::views::iota(T(0), u).begin() == T(0));
  }
  {
    const auto io = std::views::iota(T(10), u);
    assert(*io.begin() == T(10));
  }
  // Test that we return the correct type.
  {
    ASSERT_SAME_TYPE(decltype(std::views::iota(T(10))), std::ranges::iota_view<T>);
    ASSERT_SAME_TYPE(decltype(std::views::iota(T(10), u)), std::ranges::iota_view<T, U>);
  }
}

struct X {};

template <typename IntT>
concept CanDoubleWrap = requires(IntT i) { std::views::iota(std::views::iota(i)); };

constexpr bool test() {
  testType<SomeInt>(SomeInt(10));
  testType<SomeInt>(IntComparableWith(SomeInt(10)));
  testType<signed long>(IntComparableWith<signed long>(10));
  testType<unsigned long>(IntComparableWith<unsigned long>(10));
  testType<int>(IntComparableWith<int>(10));
  testType<int>(int(10));
  testType<unsigned>(unsigned(10));
  testType<unsigned>(IntComparableWith<unsigned>(10));
  testType<short>(short(10));
  testType<short>(IntComparableWith<short>(10));
  testType<unsigned short>(IntComparableWith<unsigned short>(10));

  {
    static_assert( std::is_invocable_v<decltype(std::views::iota), int>);
    static_assert(!std::is_invocable_v<decltype(std::views::iota), X>);
    static_assert( std::is_invocable_v<decltype(std::views::iota), int, int>);
    static_assert(!std::is_invocable_v<decltype(std::views::iota), int, X>);
  }
  {
    static_assert(std::same_as<decltype(std::views::iota), decltype(std::ranges::views::iota)>);
  }
  { // LWG4096: views::iota(views::iota(0)) should be rejected
    static_assert(!CanDoubleWrap<int>);
    static_assert(!CanDoubleWrap<SomeInt>);

    static_assert(!std::is_invocable_v<decltype(std::views::iota), decltype(std::views::iota(0))>);
    static_assert(!std::is_invocable_v<decltype(std::views::iota), decltype(std::views::iota(82))>);
    static_assert(!std::is_invocable_v<decltype(std::views::iota), decltype(std::views::iota(SomeInt(0)))>);
    static_assert(!std::is_invocable_v<decltype(std::views::iota), decltype(std::views::iota(SomeInt(94)))>);
  }

  return true;
}

int main(int, char**) {
  test();
  static_assert(test());

  return 0;
}
